#include "component_action_delete_on_dead.h"

/* DEPENDENCIES: Health */
/* This entity associated with this component will be deleted if it is dead. */

static void _dead_sound(ComponentActionDeleteOnDead* self, ECS* ecs);

/* Spawns a puff particle. */
static void
_puff_spawn(ComponentActionDeleteOnDead* self, ECS* ecs)
{
	ComponentSprite* sprite;
	vec3 position;
	vec2 size;

	sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, self->component.id);

	glm_vec3_copy(sprite->position, position);

	size[0] = sprite->size[1];
	size[1] = sprite->size[1];

	position[1] -= (sprite->size[1] / 2);
	position[2] += COMPONENT_ACTION_DELETE_ON_DEAD_PUFF_Z_OFFSET;

	entity_puff_init(ecs, ecs_entity_add(ecs), position, size);
}

static void
_dead_sound(ComponentActionDeleteOnDead* self, ECS* ecs)
{
	ComponentGameEntityType* gameEntityType;

	gameEntityType = ecs_component_get(ecs, ECS_COMPONENT_GAME_ENTITY_TYPE, self->component.id);

	switch (gameEntityType->type)
	{
		case GAME_ENTITY_CUPPER:
			sound_play(&ecs->game->resources.sounds[SOUND_CUPPER_KILL], SOUND_NO_PRIORITY);
			break;
		case GAME_ENTITY_CHIPPER:
			sound_play(&ecs->game->resources.sounds[SOUND_CHIPPER_KILL], SOUND_NO_PRIORITY);
			break;
		case GAME_ENTITY_CHIP:
			sound_play(&ecs->game->resources.sounds[SOUND_CHIP_KILL], SOUND_NO_PRIORITY);
			break;
		case GAME_ENTITY_BEANIE:
			sound_play(&ecs->game->resources.sounds[SOUND_BEANIE_KILL], SOUND_NO_PRIORITY);
			break;
		case GAME_ENTITY_CRUMBLER:
			sound_play(&ecs->game->resources.sounds[SOUND_CRUMBLER_KILL], SOUND_NO_PRIORITY);
			break;
		case GAME_ENTITY_CUSTARPEDO:
			sound_play(&ecs->game->resources.sounds[SOUND_CUSTARPEDO_KILL], SOUND_NO_PRIORITY);
			break;
		case GAME_ENTITY_EXCORSANT:
			sound_play(&ecs->game->resources.sounds[SOUND_EXCORSANT_KILL], SOUND_NO_PRIORITY);
			break;
		default:
			break;
	}
}

/* Ticks delete on dead component. */
void
component_action_delete_on_dead_tick(ComponentActionDeleteOnDead* self, ECS* ecs)
{
	ComponentHealth* health;

	health = ecs_component_get(ecs, ECS_COMPONENT_HEALTH, self->component.id);

	if (health->isDead)
	{
		_puff_spawn(self, ecs);
		_dead_sound(self, ecs);

		ecs_entity_delete(ecs, self->component.id);
	}
}
