#include "title.h"

static void _title_entity_init(Title* self);

/* Initializes entities for the title state. */
static void
_title_entity_init(Title* self)
{
	self->logoID = ecs_entity_add(&self->state->game->ecs);
	self->menuButtonID = ecs_entity_add(&self->state->game->ecs);
	self->textID = ecs_entity_add(&self->state->game->ecs);

	entity_sprite_init
	(
		&self->state->game->ecs,
		self->logoID,
		self->state->game->resources.textures[TEXTURE_LOGO],
		(f32*)TITLE_LOGO_SIZE,
		(f32*)TITLE_LOGO_POSITION
	);

	entity_menu_button_init
	(
		&self->state->game->ecs,
		self->menuButtonID,
		(f32*)TITLE_MENU_BUTTON_POSITION,
		MENU_BUTTON_START
	);

	entity_text_init
	(
		&self->state->game->ecs,
		self->textID,
		&self->state->game->resources.fonts[FONT_CREDITS],
		STRING_TITLE_CREDITS_TEXT,
		strlen(STRING_TITLE_CREDITS_TEXT),
		-1,
		(f32*)TITLE_CREDITS_POSITION,
		(f32*)COLOR_OPAQUE
	);
}

/* Initializes title state. */
void
title_init(Title* self, State* state)
{
	memset(self, '\0', sizeof(Title));

	self->state = state;
	
	_title_entity_init(self);

	music_play(&self->state->game->resources.music[MUSIC_TITLE], true);
}

/* Ticks title state. */
void
title_tick(Title* self)
{
	ComponentButton* button;

	button = ecs_component_get(&self->state->game->ecs, ECS_COMPONENT_BUTTON, self->menuButtonID);

	if (button->state == BUTTON_PRESSED)
	{
		sound_play(&self->state->game->resources.sounds[SOUND_SELECT], SOUND_PRIORITY);
		self->isChange = true;
	}
}
